##   MIT License
##  
## Copyright (c) 2016 Dummiesman
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
## 
## The above copyright notice and this permission notice shall be included in all
## copies or substantial portions of the Software.
## 
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
## SOFTWARE.

import sys
import struct

args = sys.argv
# check if we have enough args
if len(args) < 4:
    sys.exit("Not enough arguments! Call this script like 'python.exe heightmap_merge.py <filename>.raw <filename>.tga <size>'")

print("Heightmap Merge Script by Dummiesman\n------------------------------------")
print("If errors occur, make sure you're using a R16 RAW, and a TGA RAW BW file! Also make sure the size is correct.\n")

# parse args
raw_file = args[1]
tga_file = args[2]
terrain_size = int(args[3])

# helper functions
def read_raw(filename, size):
    file = open(filename, "rb")
    heightmap = [[0.0 for x in range(size)] for x in range(size)]
    # read RAW
    for y in range(size):
        for x in range(size):
            heightmap[x][y] = struct.unpack('H', file.read(2))[0] / 65535
    # close file and return heightmap
    file.close()
    return heightmap


def read_tgaraw(filename, size):
    file = open(filename, "rb")
    heightmap = [[0.0 for x in range(size)] for x in range(size)]
    # skip the header
    file.seek(18)
    # read TGA RAW
    for y in range(size)[::-1]:
        for x in range(size):
            heightmap[x][y] = struct.unpack('B', file.read(1))[0] / 255
    # close file and return heightmap
    file.close()
    return heightmap


def write_raw(filename, size, heightmap):
    file = open(filename, "wb")
    # write heighmap to file
    for y in range(size):
        for x in range(size):
            file.write(struct.pack('H', int(heightmap[x][y] * 65535)))
    file.close()


# main script
print("Creating array for merging operations...")
merge_heightmap = [[0.0 for x in range(terrain_size + 1)] for x in range(terrain_size + 1)]

print("Loading RAW file...")
raw_heightmap = read_raw(raw_file, terrain_size)

print("Loading TGA file...")
tga_heightmap = read_tgaraw(tga_file, terrain_size)

print("Performing merge operations...")
for x in range(terrain_size):
    for y in range(terrain_size):
        tga_val = tga_heightmap[x][y]
        raw_val = raw_heightmap[x][y]
        if tga_val > 0.0:
            merge_heightmap[x][y] = tga_val
        else:
            merge_heightmap[x][y] = raw_val
            

print("Writing a new RAW file...")
write_raw("merged.raw", terrain_size + 1, merge_heightmap)

sys.exit("Complete!")
